package model;

import main.EcologiaIO;
import controller.OccupantType;
import controller.World;

/**
 * This class simulates a carnivore.
 * 
 * @author Daniel Vedder
 * @version 30.8.2014
 */
public class Carnivore extends Animal 
{
	public static int fights_won = 1;
	public static int total_fights = 1; //Start at 1 to avoid division by zero errors

	private int[] preyPosition;
	private Direction currentDirection;
	
	//public static Genome defaultGenome = new Genome(0, 3, 15, 6, 20, 250, 11, 200, 30, 10, 1);
	public static Genome defaultGenome = new Genome(0, 3, 10, 4, 18, 200, 11, 200, 30, 10, 1);
	
	/**
	 * The constructor.
	 * @param setID
	 * @param newGenome
	 * @param myGeneration
	 * @param setX
	 * @param setY
	 * @param setEnergy
	 * @param parentID
	 */
	public Carnivore(int setID, Genome newGenome, int myGeneration, int setX, int setY, int setEnergy, int parentID) 
	{
		super(setID, OccupantType.CARNIVORE, newGenome, myGeneration, setX, setY, setEnergy, parentID);
		preyPosition = new int[2];
		currentDirection = Direction.randomDirection();
	}
	
	/**
	 * Each turn, the carnivore looks for a herbivore and moves towards it.
	 * If no herbivore can be found, it moves in a random direction.
	 */
	public void update()
	{
		super.update();
		if (!isAlive) return; //Don't do anything more if the animal is dead
		else if (energy > 50 && exhaustion > genome.getStamina() - (2 * genome.getSpeed())) return; //rest
		preyPosition = super.search(OccupantType.HERBIVORE);
		if (preyPosition != null) hunt();
		else {
			while (movesThisTurn < genome.getSpeed()) {
				boolean moved = super.move(currentDirection);
				if (!moved) currentDirection = Direction.randomDirection();
			}
		}
	}

	/**
	 * The carnivore runs toward a herbivore
	 * 
	 * --- XXX ---
	 * FIXME Buggy? Is attack() not called when preyPosition = myPosition?
	 * Somehow seems to happen when a carnivore hunts for genome.speed spaces
	 * and reaches the prey.
	 * Update: The bug is caused by a carnivore landing perchance on a herbivore
	 * it didn't 'see'.
	 * --- XXX ---
	 */
	private void hunt()
	{
		EcologiaIO.debug("Carnivore @"+x+"/"+y+" is hunting!");
		currentDirection = super.getDirection(preyPosition[0], preyPosition[1]);
		while (movesThisTurn < genome.getSpeed()) {
			boolean success = super.move(currentDirection);
			if (!success) currentDirection = currentDirection.nextDirection(true);
			//Herbivore prey = Simulator.getHerbivore(x, y);
			//if (prey != null) attack(prey);
			//FIXME I think the following line is the cause of the 'flee center' bug
			//else if (x == preyPosition[0] && y == preyPosition[1]) attack();
			else if (getDistance(preyPosition[0], preyPosition[1]) == 1
					 && movesThisTurn < genome.getSpeed())
				attack();
			else currentDirection = super.getDirection(preyPosition[0], preyPosition[1]);
		}
	}
	
	/**
	 * The carnivore has run down a herbivore and now tries to kill it
	 * XXX Warning: here be magic numbers!
	 */
	private void attack()//Herbivore prey)
	{
		EcologiaIO.debug("Carnivore @"+x+"/"+y+" is attacking a prey!");
		Herbivore prey = Simulator.getHerbivore(preyPosition[0], preyPosition[1]);//(x, y);
		if (prey == null) {
			//FIXME This should be a "can't happen", but it does...
			//Seems to be caused by phantom herbivores on the map
			//XXX I think it's fixed?
			EcologiaIO.error("Carnivore at "+x+"/"+y+" is attacking a non-existent prey!");
			return;
		}
		total_fights++;
		//Choose a fight algorithm from the methods below - currently strengthFight is used
		if (strengthFight(genome.getStrength(), prey.getGenome().getStrength())) {
			//Predators get (50+(metabolism*4))% of their preys energy
			changeEnergy((int) ((prey.getEnergy()/2)+prey.getEnergy()*(genome.getMetabolism()*0.04)));
			World.getInstance().giveNews("A Herbivore has been killed!");
			Simulator.removeAnimal(preyPosition[0], preyPosition[1], OccupantType.HERBIVORE);//(x, y, OccupantType.HERBIVORE);
			//Simulator.getField(x, y).setOccupant(OccupantType.CARNIVORE);
			//It would be more efficient to use currentDirection, but I'm not
			//sure whether it's guaranteed to be accurate
			super.move(getDirection(preyPosition[0], preyPosition[1]));
			fights_won++;
		}
		else {
			World.getInstance().giveNews("A Herbivore has won a fight!");
			//Reduce each combatants energy by their strength in the fight
			//XXX Change/remove this?
			changeEnergy(-30); //(-predStrength);
			prey.changeEnergy(-30); //(-preyStrength);
			exhaust(2);
			prey.exhaust(2);
			// if (prey.isAlive()) {
			// 	prey.extraMoves(1);
			// 	boolean fled = prey.move(Direction.randomDirection());
			// 	int ttl = 10;
			// 	while (!fled && ttl > 0) {
			// 		prey.extraMoves(1);
			// 		fled = prey.move(Direction.randomDirection());
			// 		ttl--;
			// 		if (ttl == 0) prey.setEnergy(0); //If the prey cannot escape, it dies
			// 	}
			// }
		}
		movesThisTurn = genome.getSpeed(); //an attack ends the carnivore's turn
	}

	//The following methods are various fighting algorithms
	//XXX Warning: here be magic numbers!

	/**
	 * A fight method based on the strength and current energy of the combatants.
	 */
	private boolean strengthEnergyFight(int pred_str, int pred_en, int prey_str, int prey_en)
	{
		int predStrength = pred_str+(pred_en/50);
		int preyStrength = prey_str+(prey_en/50)+super.random.nextInt(10);
		return (predStrength > preyStrength);
	}

	/**
	 * A fight method based on the strength of the combatants.
	 */
	private boolean strengthFight(int predStrength, int preyStrength)
	{
		int randomFactor = super.random.nextInt(10)-3;
		preyStrength += randomFactor;
		return (predStrength > preyStrength);
	}	

	/**
	 * A purely random fight method, based on the kill rate determined in World.
	 */
	private boolean randomFight()
	{
		return (super.random.nextInt(100) < World.getInstance().getKillRate());
	}
		

}
